#
# bind8.h       Funktionen zum schreiben der BIND8 Konfiguration
#               

CHROOTDIR=/usr/local/bind
NAMED_CONF=$CHROOTDIR/etc/named.conf
DOMAIN_NAME_DATAFILE=$CHROOTDIR/etc/db_pri.$DOMAIN_NAME
REVADDR=`/usr/local/bin/netcalc dnsrev $IP_ETH_1_IPADDR $IP_ETH_1_NETMASK`
REV_DATAFILE=$CHROOTDIR/etc/db_pri.$REVADDR
LISTEN_INTERFACE=$IP_ETH_1_IPADDR
SERIAL="`date +%Y%m%d`01"

write_resolv_conf ()
{
    echo "writing /etc/resolv.conf ..."

    (echo "#----------------------------------------------------------------------------"
     echo "# /etc/resolv.conf"
     echo "# automatically created by config.sh"
     echo "#----------------------------------------------------------------------------"
     echo "search $DOMAIN_NAME"
     echo "nameserver 127.0.0.1"
    ) >/etc/resolv.conf
}

write_named_conf ()
{
    echo "creating BIND8 configuration file ..."
    (
       echo "options {"
       echo "	directory \"/etc\";"
       echo "	dump-file \"/var/tmp/named_dump.db\";"
       echo "	pid-file \"/var/run/named.pid\";"
       echo "	statistics-file \"/var/tmp/named.stats\";"
       echo "	memstatistics-file \"/var/tmp/named.memstats\";"
       echo "	named-xfer \"/usr/sbin/named-xfer\";"
       echo "	check-names master warn;"
       echo "	check-names slave ignore;"
       echo "	heartbeat-interval $BIND8_HEARTBEAT_INTERVAL;"
       echo "	cleaning-interval $BIND8_CLEAN_INTERVAL;"
       echo "	statistics-interval 0;		// needed for telmond not to die after 1 hour"
       echo "	listen-on port 53 { 127.0.0.1; $LISTEN_INTERFACE; };"
       echo "	query-source port 53;"
       echo "	forward first;"
       echo "	forwarders {"
       for ip in $DNS_FORWARDERS
       do
          echo "		$ip;"
       done
       echo "	};"
       echo "};"
       echo
       
       echo "zone \"localhost\" {"
       echo "	type master;"
       echo " 	file \"db.localhost\";"
       echo "	check-names fail;"
       echo "	allow-query { localnets; };"
       echo "	allow-transfer { localnets; };"
       echo "	allow-update { none; };"
       echo "};"
       echo
       
       echo "zone \"0.0.127.in-addr.arpa\" {"
       echo "	type master;"
       echo "	file \"db.local\";"
       echo "	allow-query { localnets; };"
       echo "	allow-transfer { localnets; };"
       echo " 	allow-update { none; };"
       echo "};"
       echo
       
       echo "zone \"bind\" chaos {"
       echo "	type master;"
       echo "	file \"db.bind\";"
       echo " 	allow-update { none; };"
       echo "	allow-query { localhost; };"
       echo "};"
       echo
       
       echo "zone \"$DOMAIN_NAME\" {"
       echo "	type master;"
       echo "	file \"db_pri.$DOMAIN_NAME\";"
       echo "	allow-query { localnets; };"
       echo "	allow-transfer { localnets; };"
       if [ "$BIND8_DDNS_SUPPORT" = "yes" ]
       then
          echo " 	allow-update { localhost; };"
       else
          echo " 	allow-update { none; };"
       fi
       echo "};"
       echo
       
       echo "zone \"$REVADDR\" {"
       echo "	type master;"
       echo "	file \"db_pri.$REVADDR\";"
       echo "	allow-query { localnets; };"
       echo "	allow-transfer { localnets; };"
       if [ "$BIND8_DDNS_SUPPORT" = "yes" ]
       then
          echo " 	allow-update { localhost; };"
       else
          echo " 	allow-update { none; };"
       fi
       echo "};"
       echo

       idx=1
       while [ "$idx" -le "$BIND8_SLAVEZONE_N" ]
       do
          eval SLAVEDOMAIN='$BIND8_SLAVEZONE_'$idx'_DOMAIN'
          eval SLAVEMASTERS='$BIND8_SLAVEZONE_'$idx'_MASTERS'
          eval SLAVEDIALUP='$BIND8_SLAVEZONE_'$idx'_DIALUP'
          echo "zone \"$SLAVEDOMAIN\" {"
          echo "	type slave;"
          echo "	file \"db_sec.$SLAVEDOMAIN\";"
          echo "	allow-query { localnets; };"
          echo "	allow-transfer { localnets; };"
          echo "	masters {"
          for ip in $SLAVEMASTERS
          do
             echo "		$ip;"
          done
          echo "	};"
          if [ "$SLAVEDIALUP" = "yes" ]
          then
             echo "	dialup yes;"
             echo "	notify no;"
          fi
          echo "};"
          echo
          idx=`/usr/bin/expr $idx + 1`
       done

       idx=1
       while [ "$idx" -le "$DNS_FORBIDDEN_N" ]
       do
          echo "zone \"'$DNS_FORBIDDEN_'$idx\" {"
          echo "	type master;"
          echo "	file \"db.blocked\";"
          echo "	allow-query { localnets; };"
          echo "	allow-transfer { localnets; };"
          echo " 	allow-update { none; };"
          echo "};"
          echo
          idx=`/usr/bin/expr $idx + 1`
       done

       idx=1
       while [ "$idx" -le "$DNS_N" ]
       do
          set `eval echo '$DNS_'$idx`
          echo "zone \"$1\" {"
          echo "	type forward;"
          echo "	forward only;"
          echo "	forwarders { $2; };"
          echo "};"
          echo
          idx=`/usr/bin/expr $idx + 1`
       done

       echo "zone \".\" {"
       echo "	type hint;"
       echo "	file \"db.root\";"
       echo "};"
       echo
   ) > $NAMED_CONF
}

write_bind_datafiles ()
{
   if [ ! -f $DOMAIN_NAME_DATAFILE ]
   then
   echo "creating BIND8 Data file for Domain $DOMAIN_NAME ..."
      (
          echo "\$TTL 86400"
          echo "@   IN  SOA $HOSTNAME.$DOMAIN_NAME. root.$HOSTNAME.$DOMAIN_NAME. ("
          echo "       $SERIAL   ; Serial"
          echo "       3600      ; Refresh after 1 hour"
          echo "       300       ; Retry after 5 minutes"
          echo "       604800    ; Expire after 1 week"
          echo "       600 )     ; Negative Caching 10 minutes"
          echo
          echo "       IN  NS    $HOSTNAME.$DOMAIN_NAME."
          echo
          echo "localhost            IN  A     127.0.0.1"
          echo
          echo "; Hosts"
          echo ";"
          idx=1
          while [ "$idx" -le "$HOSTS_N" ]
          do
             set `eval echo '$HOST_'$idx`
             echo "$2	IN	A	$1"
             idx=`/usr/bin/expr $idx + 1`
          done
      ) > $DOMAIN_NAME_DATAFILE
   else
      echo "BIND8 Data file for Domain $DOMAIN_NAME exists ..."
   fi

   if [ ! -f $REV_DATAFILE ]
   then
   echo "creating BIND8 Data file for Domain $REVADDR ..."
      (
          echo "\$TTL 86400"
          echo "@   IN  SOA $HOSTNAME.$DOMAIN_NAME. root.$HOSTNAME.$DOMAIN_NAME. ("
          echo "       $SERIAL   ; Serial"
          echo "       3600      ; Refresh after 1 hour"
          echo "       300       ; Retry after 5 minutes"
          echo "       604800    ; Expire after 1 week"
          echo "       600 )     ; Negative Caching 10 minutes"
          echo
          echo "       IN  NS    $HOSTNAME.$DOMAIN_NAME."
          echo
   
          idx=1
          while [ "$idx" -le "$HOSTS_N" ]
          do
             set `eval echo '$HOST_'$idx`
             hostname=$2
             ipaddr=`echo $1 | sed 's/\./ /g'`
             set $ipaddr
             echo "$4      IN      PTR       $hostname.$DOMAIN_NAME."
             idx=`/usr/bin/expr $idx + 1`
          done
      ) > $REV_DATAFILE
   else
      echo "BIND8 Data file for Domain $REVADDR exists ..."
   fi
}


