/* 
 * Obtuse holelogd, a simple UNIX domain socket grab-and-throw, Opens an
 * AF_UNIX domain socket at the path specified by argv[1], (intended to be
 * /dev/log in a chrooted hole), passes anything it reads off it to the
 * socket specified by argv[2] (usually /dev/log, unless your syslog does
 * these to a nonstandard place). 
 */

char *holelogd_copyright =
"Copyright 1996 - Obtuse Systems Corporation - All rights reserved.";
char *holelogd_rcsid = "Holelogd $Id: holelogd.c,v 1.3 1996/09/15 20:36:59 beck Exp $";

/*
 * RCS information:
 * $Id: holelogd.c,v 1.3 1996/09/15 20:36:59 beck Exp $
 */


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/stat.h>

char *logname="/dev/log";

void main(int argc, char**argv) {
  fd_set readfds, cfds;
  int fds;
  struct sockaddr_un unxls, unxs;
  int listenfd;
  struct sockaddr logaddr;
  char *holename=NULL;
  int logfd;
  int boom = 0;

  switch (argc) {
  case 3:
    logname=argv[2];
    /* fall through */ 
  case 2:
    holename=argv[1];
    break;
  default:
    fprintf(stderr, "usage: %s /wherever/dev/log [/dev/log]\n", argv[0]);
    exit(1);
  }

  /* daemonize ourselves */

   if (fork() != 0) {
      exit(0);
   }
   setsid();

  /* open unix listen socket in hole */

  (void)unlink(holename);   
  unxls.sun_family = AF_UNIX;
  (void) strncpy(unxls.sun_path, holename, sizeof unxls.sun_path);
  listenfd = socket(AF_UNIX, SOCK_STREAM, 0);
  if (listenfd < 0 || bind(listenfd, (struct sockaddr *) &unxls,
			sizeof(unxls.sun_family)+strlen(unxls.sun_path)) < 0 ||
      chmod(holename, 0666) < 0 || listen(listenfd, 5) < 0) {
    syslog(LOG_ERR, "Can't create AF_UNIX listen socket %s (%m) - exiting",
	   holename);  
    exit(1);
  }

  /* open connection to real log socket */

  logaddr.sa_family = AF_UNIX;
  (void) strncpy(logaddr.sa_data, logname, sizeof(logaddr.sa_data));
  logfd = socket(AF_UNIX, SOCK_STREAM, 0);
  if (logfd < 0) {
    syslog(LOG_ERR, "Can't create AF_UNIX socket (%m) - exiting");
    exit(1);
  }
  if (connect(logfd, &logaddr, sizeof(logaddr.sa_family) + 
	      strlen(logaddr.sa_data)) < 0) {
    syslog(LOG_ERR, "Can't connect to socket %s (%m) - exiting",
	   logaddr.sa_data
	   );
    exit(1);
  }
  
  /* we should now be able to throw things at logfd
   * so now we accept connections on the hole socket, 
   * and throw anything we read off them to logfd.
   */
  
  FD_ZERO(&cfds);
  FD_SET(listenfd, &cfds);
  boom=0;

  while(1) {
    memcpy(&readfds, &cfds, sizeof(fd_set)); 
    fds = select(FD_SETSIZE, &readfds, NULL, NULL, NULL);
    if (fds < 0) {
      if ( errno != EINTR ) {
	syslog(LOG_ERR, "Select failed! (%m)"); 
      }
      if (boom > 10) {
	syslog(LOG_CRIT, "Select continuously failing! - exiting"); 
	exit(1);
      }
    }
    else {
      boom=0;
      if (fds > 0) {
	int fd;
	for (fd = 0; fd < FD_SETSIZE; fd++) {
	  if (FD_ISSET(fd, &readfds)) {
	    if (fd == listenfd) {
	      int len;
	      len = sizeof(unxs);
	      fd = accept(listenfd, (struct sockaddr *)&unxs, &len);
	      if (fd >= 0) {
		FD_SET(fd, &cfds);
	      }
	    }
	    else {
	      int i;
	      char  buf[1024];
	      i = read(fd, buf, 1024);
	      if (i < 0) {
		syslog(LOG_ERR, "Read error on connected socket (%m)");
	      }
	      else if (i > 0) {
		int j = 0;
		while (j < i) {
		  int k;
		  k=write(logfd, buf+j, i-j);
		  if (k < 0) {
		    syslog(LOG_ERR, "Write failed to socket %s, (%m) - exiting",
			   logname);
		    exit(1);
		  }
		  j+=k;
		}
	      }
	      else {
		/* EOF */
		close(fd);
		FD_CLR(fd, &cfds);
	      }
	    }
	  }
	}
      }
    }
  }
}
